"use strict";

exports.__esModule = true;
exports.useNumberInput = useNumberInput;

var _chakraUiCounter = require("@hackr/chakra-ui-counter");

var _chakraUiHooks = require("@hackr/chakra-ui-hooks");

var _chakraUiUtils = require("@hackr/chakra-ui-utils");

var _react = require("react");

var _useSpinner = require("./use-spinner");

var _utils = require("./utils");

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

/**
 * React hook that implements the WAI-ARIA Spin Button widget
 * and used to create numeric input fields.
 *
 * It returns prop getters you can use to build your own
 * custom number inputs.
 *
 * @see WAI-ARIA https://www.w3.org/TR/wai-aria-practices-1.1/#spinbutton
 * @see Docs     https://www.chakra-ui.com/useNumberInput
 * @see WHATWG   https://html.spec.whatwg.org/multipage/input.html#number-state-(type=number)
 */
function useNumberInput(props) {
  if (props === void 0) {
    props = {};
  }

  var _props = props,
      _props$focusInputOnCh = _props.focusInputOnChange,
      focusInputOnChange = _props$focusInputOnCh === void 0 ? true : _props$focusInputOnCh,
      _props$clampValueOnBl = _props.clampValueOnBlur,
      clampValueOnBlur = _props$clampValueOnBl === void 0 ? true : _props$clampValueOnBl,
      _props$keepWithinRang = _props.keepWithinRange,
      keepWithinRange = _props$keepWithinRang === void 0 ? true : _props$keepWithinRang,
      _props$min = _props.min,
      min = _props$min === void 0 ? _chakraUiUtils.minSafeInteger : _props$min,
      _props$max = _props.max,
      max = _props$max === void 0 ? _chakraUiUtils.maxSafeInteger : _props$max,
      _props$step = _props.step,
      stepProp = _props$step === void 0 ? 1 : _props$step,
      isReadOnly = _props.isReadOnly,
      isDisabled = _props.isDisabled,
      getAriaValueText = _props.getAriaValueText,
      isInvalid = _props.isInvalid,
      _props$pattern = _props.pattern,
      pattern = _props$pattern === void 0 ? "[0-9]*(.[0-9]+)?" : _props$pattern,
      _props$inputMode = _props.inputMode,
      inputMode = _props$inputMode === void 0 ? "decimal" : _props$inputMode,
      allowMouseWheel = _props.allowMouseWheel,
      id = _props.id,
      onChangeProp = _props.onChange,
      precision = _props.precision,
      htmlProps = _objectWithoutPropertiesLoose(_props, ["focusInputOnChange", "clampValueOnBlur", "keepWithinRange", "min", "max", "step", "isReadOnly", "isDisabled", "getAriaValueText", "isInvalid", "pattern", "inputMode", "allowMouseWheel", "id", "onChange", "precision"]);
  /**
   * Leverage the `useCounter` hook since it provides
   * the functionality to `increment`, `decrement` and `update`
   * counter values
   */


  var counter = (0, _chakraUiCounter.useCounter)(props);
  var updateFn = counter.update,
      incrementFn = counter.increment,
      decrementFn = counter.decrement;
  /**
   * Keep track of the focused state of the input,
   * so user can this to change the styles of the
   * `spinners`, maybe :)
   */

  var _useBoolean = (0, _chakraUiHooks.useBoolean)(),
      isFocused = _useBoolean[0],
      setFocused = _useBoolean[1];

  var inputRef = (0, _react.useRef)(null);
  var isInteractive = !(isReadOnly || isDisabled);
  var increment = (0, _react.useCallback)(function (step) {
    if (step === void 0) {
      step = stepProp;
    }

    if (isInteractive) {
      incrementFn(step);
    }
  }, [incrementFn, isInteractive, stepProp]);
  var decrement = (0, _react.useCallback)(function (step) {
    if (step === void 0) {
      step = stepProp;
    }

    if (isInteractive) {
      decrementFn(step);
    }
  }, [decrementFn, isInteractive, stepProp]);
  /**
   * Leverage the `useSpinner` hook to spin the input's value
   * when long press on the up and down buttons.
   *
   * This leverages `setInterval` internally
   */

  var spinner = (0, _useSpinner.useSpinner)(increment, decrement);
  /**
   * The `onChange` handler filters out any character typed
   * that isn't floating point compatible.
   */

  var onChange = (0, _react.useCallback)(function (event) {
    var valueString = event.target.value.split("").filter(_utils.isFloatingPointNumericCharacter).join("");
    updateFn(valueString);
  }, [updateFn]);
  var onKeyDown = (0, _react.useCallback)(function (event) {
    /**
     * only allow valid numeric keys
     */
    if (!(0, _utils.isValidNumericKeyboardEvent)(event)) {
      event.preventDefault();
    }
    /**
     * Keyboard Accessibility
     *
     * We want to increase or decrease the input's value
     * based on if the user the arrow keys.
     *
     * @see https://www.w3.org/TR/wai-aria-practices-1.1/#keyboard-interaction-17
     */


    var stepFactor = getStepFactor(event) * stepProp;
    var eventKey = (0, _chakraUiUtils.normalizeEventKey)(event);
    var keyMap = {
      ArrowUp: function ArrowUp() {
        return increment(stepFactor);
      },
      ArrowDown: function ArrowDown() {
        return decrement(stepFactor);
      },
      Home: function Home() {
        return updateFn(min);
      },
      End: function End() {
        return updateFn(max);
      }
    };
    var action = keyMap[eventKey];

    if (action) {
      event.preventDefault();
      action(event);
    }
  }, [updateFn, decrement, increment, max, min, stepProp]);

  var getStepFactor = function getStepFactor(event) {
    var ratio = 1;

    if (event.metaKey || event.ctrlKey) {
      ratio = 0.1;
    }

    if (event.shiftKey) {
      ratio = 10;
    }

    return ratio;
  };
  /**
   * If user would like to use a human-readable representation
   * of the value, rather than the value itself they can pass `getAriaValueText`
   *
   * @see https://www.w3.org/TR/wai-aria-practices-1.1/#wai-aria-roles-states-and-properties-18
   * @see https://www.w3.org/TR/wai-aria-1.1/#aria-valuetext
   */


  var _getAriaValueText = function _getAriaValueText() {
    var text = getAriaValueText == null ? void 0 : getAriaValueText(counter.value);

    if (!(0, _chakraUiUtils.isNull)(text)) {
      return text;
    }

    var defaultText = counter.value.toString(); // empty string is an invalid ARIA attribute value

    return !defaultText ? undefined : defaultText;
  };

  var ariaValueText = _getAriaValueText();
  /**
   * Function that clamps the input's value on blur
   */


  var validateAndClamp = (0, _react.useCallback)(function () {
    var next = counter.value;
    if (next === "") return;

    if (counter.valueAsNumber < min) {
      next = min;
    }

    if (counter.valueAsNumber > max) {
      next = max;
    }
    /**
     * `counter.cast` does 2 things:
     *
     * - sanitize the value by using parseFloat and some Regex
     * - used to round value to computed precision or decimal points
     */


    if (counter.value !== next) {
      counter.cast(next);
    }
  }, [counter, max, min]);
  var onBlur = (0, _react.useCallback)(function () {
    setFocused.off();

    if (clampValueOnBlur) {
      validateAndClamp();
    }
  }, [clampValueOnBlur, setFocused, validateAndClamp]);
  var focusInput = (0, _react.useCallback)(function () {
    if (focusInputOnChange && inputRef.current) {
      (0, _chakraUiUtils.focus)(inputRef.current);
    }
  }, [focusInputOnChange]);
  var spinUp = (0, _react.useCallback)(function (event) {
    event.preventDefault();
    spinner.up();
    focusInput();
  }, [focusInput, spinner]);
  var spinDown = (0, _react.useCallback)(function (event) {
    event.preventDefault();
    spinner.down();
    focusInput();
  }, [focusInput, spinner]);
  var pointerDown = _chakraUiUtils.isBrowser && !!document.documentElement.ontouchstart ? "onTouchStart" : "onMouseDown";
  (0, _chakraUiHooks.useEventListener)("wheel", function onWheel(event) {
    var isInputFocused = document.activeElement === inputRef.current;
    if (!allowMouseWheel || !isInputFocused) return;
    event.preventDefault();
    var stepFactor = getStepFactor(event) * stepProp;
    var direction = Math.sign(event.deltaY);

    if (direction === -1) {
      increment(stepFactor);
    } else if (direction === 1) {
      decrement(stepFactor);
    }
  }, inputRef.current);
  var getIncrementButtonProps = (0, _react.useCallback)(function (props, ref) {
    var _extends2;

    if (props === void 0) {
      props = {};
    }

    if (ref === void 0) {
      ref = null;
    }

    var disabled = isDisabled || keepWithinRange && counter.isAtMax;
    return _extends({}, props, (_extends2 = {
      ref: ref,
      role: "button",
      tabIndex: -1
    }, _extends2[pointerDown] = (0, _chakraUiUtils.callAllHandlers)(props[pointerDown], spinUp), _extends2.onMouseUp = (0, _chakraUiUtils.callAllHandlers)(props.onMouseUp, spinner.stop), _extends2.onMouseLeave = (0, _chakraUiUtils.callAllHandlers)(props.onMouseUp, spinner.stop), _extends2.onTouchEnd = (0, _chakraUiUtils.callAllHandlers)(props.onTouchEnd, spinner.stop), _extends2.disabled = disabled, _extends2["aria-disabled"] = (0, _chakraUiUtils.ariaAttr)(disabled), _extends2));
  }, [pointerDown, counter.isAtMax, keepWithinRange, spinUp, spinner.stop]);
  var getDecrementButtonProps = (0, _react.useCallback)(function (props, ref) {
    var _extends3;

    if (props === void 0) {
      props = {};
    }

    if (ref === void 0) {
      ref = null;
    }

    var disabled = isDisabled || keepWithinRange && counter.isAtMin;
    return _extends({}, props, (_extends3 = {
      ref: ref,
      role: "button",
      tabIndex: -1
    }, _extends3[pointerDown] = (0, _chakraUiUtils.callAllHandlers)(props[pointerDown], spinDown), _extends3.onMouseLeave = (0, _chakraUiUtils.callAllHandlers)(props.onMouseLeave, spinner.stop), _extends3.onMouseUp = (0, _chakraUiUtils.callAllHandlers)(props.onMouseUp, spinner.stop), _extends3.onTouchEnd = (0, _chakraUiUtils.callAllHandlers)(props.onTouchEnd, spinner.stop), _extends3.disabled = disabled, _extends3["aria-disabled"] = (0, _chakraUiUtils.ariaAttr)(disabled), _extends3));
  }, [pointerDown, counter.isAtMin, keepWithinRange, spinDown, spinner.stop]);
  var getInputProps = (0, _react.useCallback)(function (props, ref) {
    var _props$inputMode2;

    if (props === void 0) {
      props = {};
    }

    if (ref === void 0) {
      ref = null;
    }

    return _extends({}, props, {
      id: id,
      ref: (0, _chakraUiUtils.mergeRefs)(inputRef, ref),
      value: counter.value,
      role: "spinbutton",
      type: "text",
      inputMode: (_props$inputMode2 = props["inputMode"]) != null ? _props$inputMode2 : inputMode,
      pattern: pattern,
      "aria-valuemin": min,
      "aria-valuemax": max,
      "aria-disabled": isDisabled,
      "aria-valuenow": Number.isNaN(counter.valueAsNumber) ? undefined : counter.valueAsNumber,
      "aria-invalid": isInvalid || counter.isOutOfRange,
      "aria-valuetext": ariaValueText,
      readOnly: isReadOnly,
      disabled: isDisabled,
      autoComplete: "off",
      autoCorrect: "off",
      onChange: (0, _chakraUiUtils.callAllHandlers)(props.onChange, onChange),
      onKeyDown: (0, _chakraUiUtils.callAllHandlers)(props.onKeyDown, onKeyDown),
      onFocus: (0, _chakraUiUtils.callAllHandlers)(props.onFocus, setFocused.on),
      onBlur: (0, _chakraUiUtils.callAllHandlers)(props.onBlur, onBlur)
    });
  }, [inputMode, pattern, ariaValueText, counter.isOutOfRange, counter.value, counter.valueAsNumber, id, isDisabled, isInvalid, isReadOnly, max, min, onBlur, onChange, onKeyDown, setFocused.on]);
  return {
    value: counter.value,
    valueAsNumber: counter.valueAsNumber,
    isFocused: isFocused,
    isDisabled: isDisabled,
    isReadOnly: isReadOnly,
    getIncrementButtonProps: getIncrementButtonProps,
    getDecrementButtonProps: getDecrementButtonProps,
    getInputProps: getInputProps,
    htmlProps: htmlProps
  };
}
//# sourceMappingURL=use-number-input.js.map